import random
import sys

class Wall(object):
    iswall = True

class Open(object):
    iswall = False

class MazeBuilder(object):
    def ascii_draw(self, cave):
        width = len(cave[0]) + 2
        sys.stdout.write('#' * width + '\n')

        for row in cave:
            sys.stdout.write('#')
            for cell in row:
                if cell.iswall:
                    sys.stdout.write('#')
                else:
                    sys.stdout.write('.')
            sys.stdout.write('#\n')

        sys.stdout.write('#' * width + '\n')

    # Given an adjoining wall (a wall which separates two labyrinth cells)
    # set a and b to point to the cell indices which are separated. Used by
    # labyrinth_gen().
    def lab_get_adjoin(self, y, x):
        if x % 2 == 0:
            p1 = (y - 1, x)
            p2 = (y + 1, x)
        else:
            p1 = (y, x - 1)
            p2 = (y, x + 1)
        return (p1, p2)
    
    # Build a labyrinth level.
    def generate_maze(self, h=15, w=47):
        assert h % 2 == 1
        assert w % 2 == 1

        # This will be a two-dimensional list of our cells.
        cave = []

        # This will be a map giving each unconnected cell "area" its own color.
        sets = {}

        # This is a list of all the tile coords, which we will later randomize.
        walls = []
    
        # Allocate all the grids, start them off as walls.
        for y in range(0, h):
            row = []
            for x in range(0, w):
                row.append(Wall())
                walls.append((y, x))
                sets[(y, x)] = y + x * w
            cave.append(row)

        # Make the areas where x and y are both even open.
        for y in range(0, h, 2):
            for x in range(0, w, 2):
                cave[y][x] = Open()
    
        # Randomize the wall list.
        random.shuffle(walls)
    
        # For each adjoining wall, look at the cells it divides. If they aren't
        # in the same set, remove the wall and join their sets.
        #
        # This is a randomized version of Kruskal's algorithm.
        for wall in walls:
            y, x = wall
    
            # If this cell isn't an adjoining wall, skip it */
            if (x < 1 and y < 1) or (x > w - 2 and y > h - 2): continue
            if (x % 2 == y % 2): continue
        
            # Figure out which cells are separated by this wall */
            p1, p2 = self.lab_get_adjoin(y, x)

            # If the cells aren't connected, kill the wall and join the sets */
            #continue
            if sets[p1] != sets[p2]:
                cave[y][x] = Open()

                # Now change all the cells of p2's color to p1's color.
                replace = sets[p2]
                for y2 in range(0, h):
                    for x2 in range(0, w):
                        if sets[(y2, x2)] == replace:
                            sets[(y2, x2)] = sets[p1]
        return cave

if __name__ == "__main__":
    b = MazeBuilder()
    cave = b.generate_maze(15, 31)
    b.ascii_draw(cave)
